package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesLabelTemplateService;
import cn.estsh.i3plus.ext.mes.api.base.IMesLoadingListService;
import cn.estsh.i3plus.ext.mes.api.base.IMesPartShippingGroupService;
import cn.estsh.i3plus.ext.mes.apiservice.config.AuthUtilExt;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.print.IPrintTemplateStrategyService;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.ext.mes.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.mes.api.iservice.base.IConfigService;
import cn.estsh.i3plus.mes.api.iservice.busi.ISyncFuncService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesConfig;
import cn.estsh.i3plus.pojo.mes.bean.MesLabelTemplate;
import cn.estsh.i3plus.pojo.mes.bean.MesPartShippingGroup;
import cn.estsh.i3plus.pojo.mes.bean.edi.cd.*;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesLoadingList;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesLoadingListDetail;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesShippingOrderManagement;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesShippingOrderManagementDetail;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.repository.MesLoadingListDetailRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesShippingOrderManagementDetailRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesShippingOrderManagementRepository;
import cn.estsh.i3plus.pojo.mes.repository.edi.cd.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.SpringContextsUtil;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import cn.hutool.core.date.DateTime;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.elasticsearch.common.recycler.Recycler;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@Service
@Slf4j
public class MesLoadingListService extends BaseMesService<MesLoadingList> implements IMesLoadingListService {

    @Autowired
    private MesLoadingListDetailService detailService;
    @Autowired
    private MesShippingOrderManagementRepository shippingOrderManagementRepository;
    @Autowired
    private MesShippingOrderManagementDetailRepository shippingOrderManagementDetailRDao;
    @Autowired
    private MesLoadingListDetailRepository listDetailRepository;
    @Autowired
    private IMesPartShippingGroupService mesPartShippingGroupService;
    @Autowired
    private ISyncFuncService syncFuncService;
    @Autowired
    private IConfigService configService;

    @Autowired
    private IMesLabelTemplateService labelTemplateService;
    @Autowired
    private MesCimVolvoJisRackIdDetailRepository volvoJisRackIdDetailRDao;
    @Autowired
    private MesCimVolvoJisRackIdRepository volvoJisRackIdRDao;
    @Autowired
    private MesCimVolvoLdjisRepository volvoLdjisRDao;
    @Autowired
    private MesCimVolvoAsnRepository volvoAsnRDao;
    @Autowired
    private MesCimVolvoAsnDetailRepository volvoAsnDetailRDao;

    @Override
    public MesLoadingList insert(MesLoadingList bean) {
        //插入前构造并校验数据
        onInsertBean(bean);

        MesShippingOrderManagement shippingOrderManagement = bean.getShippingOrderNoList().get(0);
        MesPartShippingGroup partShippingGroup = mesPartShippingGroupService.getMesPartShippingGroup(shippingOrderManagement.getOrganizeCode(),shippingOrderManagement.getShippingGroupCode());

        if (StringUtils.isEmpty(partShippingGroup)) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】零件发运组数据为空，请检查数据", shippingOrderManagement.getShippingCode())
                    .build();
        }

        if (StringUtils.isEmpty(partShippingGroup.getIsLoading()) || partShippingGroup.getIsLoading() == 0) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】零件发运组是否有装车不能为空，请检查数据", shippingOrderManagement.getShippingCode())
                    .build();
        }
        if (partShippingGroup.getIsLoading() == MesCommonConstant.FALSE_INTEGER) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】零件发运组是否有装车配置为否，则不需要生成装车单，请检查数据", shippingOrderManagement.getShippingCode())
                    .build();
        }

        if (StringUtils.isEmpty(partShippingGroup.getIsFirstInstall()) || partShippingGroup.getIsFirstInstall() == 0) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】零件发运组是否先装车不能为空，请检查数据", shippingOrderManagement.getShippingCode())
                    .build();
        }
        if (StringUtils.isEmpty(partShippingGroup.getIsEnableMemory()) || partShippingGroup.getIsEnableMemory() == 0) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】零件发运组是否启用记忆不能为空，请检查数据", shippingOrderManagement.getShippingCode())
                    .build();
        }

        //是否逆转
        if (partShippingGroup.getIsReverse() == MesCommonConstant.TRUE_INTEGER && partShippingGroup.getReverseQuantity() < 1) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("【%s】零件发运组配置是否逆转为【是】，逆转数不能小于1，请检查数据", shippingOrderManagement.getShippingCode())
                    .build();
        }


        //生成工单号
        GenSerialNoModel genSerialNoModel = new GenSerialNoModel(MesCommonConstant.LOADING_LIST_CODE);
//        genSerialNoModel.setPartNo(bean.getPartNo());
        List<String> resultList = syncFuncService.syncSerialNo(genSerialNoModel.organizeCode(bean.getOrganizeCode()), bean.getCreateUser(), bean.getOrganizeCode(), 1).getResultList();

        bean.setPartShippingGroupCode(partShippingGroup.getShippingGroupCode());
        bean.setShippingGroupName(partShippingGroup.getShippingGroupName());
        bean.setStatus(MesExtEnumUtil.LOADING_DETAIL_ORDER_STATUS.CREATE.getValue());
        bean.setOrderCode(resultList.get(0));
        bean.setIsFirstInstall(partShippingGroup.getIsFirstInstall());
        bean.setIsEnableMemory(partShippingGroup.getIsEnableMemory());
//        bean.setSeq(bean.getOrderCode());
        bean.setStatus(MesExtEnumUtil.LOADING_ORDER_STATUS.CREATE.getValue());
        String customerCode = bean.getShippingOrderNoList().get(0).getCustCode();
        String customerName = bean.getShippingOrderNoList().get(0).getCustName();
        String customerOrganizeCode = bean.getShippingOrderNoList().get(0).getCustOrganizeCode();
        String customerOrganizeName = bean.getShippingOrderNoList().get(0).getCustOrganizeName();
        bean.setCustCode(customerCode);
        bean.setCustName(customerName);
        bean.setCustOrganizeCode(customerOrganizeCode);
        bean.setCustOrganizeName(customerOrganizeName);
        bean = baseRDao.insert(bean);

        List<MesLoadingListDetail> detailList = new ArrayList<>();
        MesLoadingList finalBean = bean;

        bean.getShippingOrderNoList().forEach(k -> {
            MesLoadingListDetail detail = new MesLoadingListDetail();

            k.setOrderCode(finalBean.getOrderCode());
            k.setLoadingListId(StringUtil.isEmpty(finalBean.getId()) ? 0 : finalBean.getId());

            BeanUtils.copyProperties(k, detail, "id");
            detail.setStatus(MesExtEnumUtil.LOADING_ORDER_DETAIL_SHIPPING_STATUS.PUBLISHED.getValue());
            detail.setPid(finalBean.getId());
            detail.setShippingId(k.getId());
            ConvertBean.serviceModelInitialize(detail, AuthUtil.getSessionUser().getUserName());
            ConvertBean.serviceModelUpdate(k, AuthUtil.getSessionUser().getUserName());
            detailList.add(detail);
        });

        shippingOrderManagementRepository.saveAll(bean.getShippingOrderNoList());
        detailService.insertBatch(detailList);

        return bean;
    }

    @Override
    public void deleteWeaklyByIds(Long[] ids, String userName) {
        String organizeCode = AuthUtilExt.getOrganizeCode();
        for (Long id : ids) {
            DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getNumEqualPack(id,"id",packBean);
            MesLoadingList bean = baseRDao.getByProperty(packBean);
            if (StringUtils.isEmpty(bean)) continue;
            DdlPackBean detailPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
            DdlPreparedPack.getNumEqualPack(bean.getId(), "pid", detailPackBean);
            List<MesLoadingListDetail> detailDeleteList = detailService.findAll(detailPackBean);
            detailDeleteList.forEach(k -> {
                k.setIsDeleted(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
                ConvertBean.serviceModelUpdate(k, AuthUtil.getSessionUser().getUserName());
            });
            bean.setIsDeleted(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
            ConvertBean.serviceModelUpdate(bean, AuthUtil.getSessionUser().getUserName());
            baseRDao.update(bean);
            detailService.updates(detailDeleteList);
        }
    }

    @Override
    protected void onInsertBean(MesLoadingList item) {

        List<String> shippingOrderNoList = new ArrayList<>();
        List<String> listDetailByShippingCodeList = new ArrayList<>();
        // 数据校验
        ValidatorBean.checkNotNull(item.getTruckNo(), "车牌号不能为空");
        ValidatorBean.checkNotNull(item.getWeather(), "天气不能为空");
        ValidatorBean.checkNotNull(item.getShippingOrderNoList(), "发运单不能为空");
        item.getShippingOrderNoList().forEach(k -> {
            if (k.getStatus() != MesExtEnumUtil.SHIPPING_ORDER_STATUS.CREATE.getValue() && k.getStatus() != MesExtEnumUtil.SHIPPING_ORDER_STATUS.PUBLISHED.getValue()) {
                shippingOrderNoList.add(k.getShippingCode());
            }

            DdlPackBean partPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
            DdlPreparedPack.getNumEqualPack(k.getId(), "shippingId", partPackBean);
            boolean flg = listDetailRepository.isExitByHql(partPackBean);
            if (flg) {
                listDetailByShippingCodeList.add(k.getShippingCode());
            }
        });

        if (!CollectionUtils.isEmpty(shippingOrderNoList)) {
            throw ImppExceptionBuilder.newInstance()
                    .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                    .setErrorDetail(String.format("发运单【%s】状态不是【%s】或【%s】,请重新选择数据", shippingOrderNoList, MesExtEnumUtil.SHIPPING_ORDER_STATUS.CREATE.getValue(),
                            MesExtEnumUtil.SHIPPING_ORDER_STATUS.PUBLISHED.getValue()))
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .build();
        }
        if (!CollectionUtils.isEmpty(listDetailByShippingCodeList)) {
            throw ImppExceptionBuilder.newInstance()
                    .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                    .setErrorDetail(String.format("发运单【%s】生成装车单,请重新选择数据", listDetailByShippingCodeList))
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .build();
        }

//        DdlPackBean partPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
//        DdlPreparedPack.getStringEqualPack(item.getTruckNo(), "truckNo", partPackBean);
//        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.LOADING_ORDER_STATUS.CREATE.getValue(), "status", partPackBean);
//        boolean flg = baseRDao.isExitByHql(partPackBean);
//        if (flg) {
//            throw ImppExceptionBuilder.newInstance()
//                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
//                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
//                    .setErrorDetail("车牌号【%s】已经生成创建状态的装车单，请检查数据", item.getTruckNo())
//                    .build();
//        }

    }

    @Override
    protected void setPackQueryBean(MesLoadingList bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getOrderCode(), "orderCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCustCode(), "custCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCustOrganizeCode(), "custOrganizeCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getTruckNo(), "truckNo", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getRemark(), "remark", packBean);
    }

    @Override
    public MesLoadingList getMesLoadingListByShippingCode(String organizeCode, String shippingCode) {
        MesLoadingListDetail mesLoadingListDetail = detailService.getMesLoadingListDetail(organizeCode,shippingCode);
        if(Objects.isNull(mesLoadingListDetail)){
            return null;
        }
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getNumEqualPack(mesLoadingListDetail.getPid(),"id",packBean);
        return baseRDao.getByProperty(packBean);
    }

    private List<MesLoadingList> getMesLoadingListByIds(String organizeCode, List<Long> ids) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(ids,"id",packBean);
        return baseRDao.findByHqlWhere(packBean);
    }

    @Override
    public ResultBean doMesMesLoadingListPrint(List<Long> ids, boolean reprint) {
        String organizeCode = AuthUtilExt.getOrganizeCode();
        MesConfig config = configService.getConfigByCfgCode(MesExtConstWords.LOADING_LIST_PRINT_TEMPLATE, organizeCode);
        if (null == config || StringUtils.isEmpty(config.getCfgValue())) {
            MesException.throwBusiException("未配置装车单全局打印配置[%s]", MesExtConstWords.LOADING_LIST_PRINT_TEMPLATE);
        }

        MesLabelTemplate labelTemplate = labelTemplateService.getLabelTemplate(config.getCfgValue(), organizeCode);
        if (labelTemplate == null || StringUtils.isEmpty(labelTemplate.getMethodCode())) {
            MesException.throwBusiException("装车单打印模版[%s]未配置策略方法", config.getCfgValue());
        }

        List<MesLoadingList> loadingLists = getMesLoadingListByIds(organizeCode, ids);
        if (CollectionUtils.isEmpty(loadingLists)) {
            return ResultBean.success("不存在需要打印的装车单").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }

        log.info("工厂{}装车单打印 --- 获取到打印数据:{}条 --- loadingLists:{} --- {}",
                organizeCode, loadingLists.size(), JSONObject.toJSONString(loadingLists), Thread.currentThread().getName());

        IPrintTemplateStrategyService printStrategyService = (IPrintTemplateStrategyService) SpringContextsUtil.getBean(labelTemplate.getMethodCode());
        ResultBean resultBean = printStrategyService.execute(loadingLists, reprint);

        log.info("工厂{}排序发运打印 --- 封装打印数据完成 --- {}", organizeCode, Thread.currentThread().getName());

        return resultBean;
    }

    @Override
    public void doSubmit(List<Long> ids) {
        List<MesLoadingList> loadingLists = getMesLoadingListByIds(ids);
        for (MesLoadingList loadingList : loadingLists) {
            if (loadingList.getStatus() > MesExtEnumUtil.MES_LOADING_STATUS.PUBLISH.getValue()) {
                throw ImppExceptionBuilder.newInstance()
                        .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                        .setErrorDetail(String.format("装车单已扫描，不能再确认或取消确认！"))
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .build();
            }
        }

        for (MesLoadingList loading : loadingLists) {
            List<MesShippingOrderManagement> orderManagements = getMesShippingOrderManagement(loading);
            if (loading.getStatus() == MesExtEnumUtil.MES_LOADING_STATUS.CREATE.getValue()) {
                loading.setStatus(MesExtEnumUtil.MES_LOADING_STATUS.PUBLISH.getValue());
                for (MesShippingOrderManagement orderManagement : orderManagements) {
                    orderManagement.setStatus(MesExtEnumUtil.SHIPPING_ORDER_STATUS.LOADING.getValue());
                    shippingOrderManagementRepository.update(orderManagement);
                }
                baseRDao.update(loading);
            } else if (loading.getStatus() == MesExtEnumUtil.MES_LOADING_STATUS.PUBLISH.getValue()) {
                loading.setStatus(MesExtEnumUtil.MES_LOADING_STATUS.CREATE.getValue());
                for (MesShippingOrderManagement orderManagement : orderManagements) {
                    orderManagement.setStatus(MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPED.getValue());
                    shippingOrderManagementRepository.update(orderManagement);
                }
                baseRDao.update(loading);
            }
        }
    }

    @Override
    public void doCreateVolvoAsn(List<Long> ids) {
        String organizeCode = AuthUtilExt.getOrganizeCode();
        List<MesLoadingList> loadingLists = getMesLoadingListByIds(ids);
        for (MesLoadingList loading : loadingLists) {
            if (loading.getStatus() != MesExtEnumUtil.MES_LOADING_STATUS.SCANNED.getValue()) {
                throw ImppExceptionBuilder.newInstance()
                        .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                        .setErrorDetail(String.format("装车单【%s】未执行装车校验！", loading.getOrderCode()))
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .build();
            }
        }
        for (MesLoadingList loading : loadingLists) {
            DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getNumEqualPack(loading.getId(), "pid", packBean);
            List<MesLoadingListDetail> details = listDetailRepository.findByHqlWhere(packBean);
            if (CollectionUtils.isEmpty(details)) {
                throw ImppExceptionBuilder.newInstance()
                        .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                        .setErrorDetail(String.format("装车单【%s】明细为空！", loading.getOrderCode()))
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .build();
            }
            MesCimVolvoJisRackId jisRackId = null;
            for (MesLoadingListDetail detail : details) {
                DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
                DdlPreparedPack.getStringEqualPack(detail.getShippingCode(), "shippingCode", ddlPackBean);
                MesShippingOrderManagement orderManagement = shippingOrderManagementRepository.getByProperty(ddlPackBean);
                if (orderManagement != null) {
                    ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
                    DdlPreparedPack.getNumEqualPack(orderManagement.getId(), "pid", ddlPackBean);
                    List<MesShippingOrderManagementDetail> orderManagementDetails = shippingOrderManagementDetailRDao.findByHqlWhere(ddlPackBean);
                    for (MesShippingOrderManagementDetail orderManagementDetail : orderManagementDetails) {
                        if (orderManagementDetail.getSourceId() != null) {
                            MesCimVolvoJisRackIdDetail rackIdDetail = volvoJisRackIdDetailRDao.getById(orderManagementDetail.getSourceId());
                            if (rackIdDetail != null) {
                                jisRackId = volvoJisRackIdRDao.getById(rackIdDetail.getJisRackIdFid());
                                break;
                            }
                        }
                    }
                }
                if (jisRackId != null) {
                    break;
                }
            }
            if (jisRackId == null) {
                continue;
            }
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(jisRackId.getConsignmentReference(), "consignmentReference", ddlPackBean);
            MesCimVolvoLdjis volvoLdjis = volvoLdjisRDao.getByProperty(ddlPackBean);
            if (volvoLdjis == null) {
                continue;
            }

            MesCimVolvoAsn volvoAsn = new MesCimVolvoAsn();
            volvoAsn.setOrganizeCode(organizeCode);
            volvoAsn.setAsnDate(TimeTool.getNowTime("yyyy-MM-dd HH:mm:ss.SSS"));
            volvoAsn.setAsnTime(TimeTool.getNowTime("yyyy-MM-dd HH:mm:ss.SSS"));
            volvoAsn.setStatus(10);
            volvoAsn.setMessage(volvoLdjis.getInterchangeHeader());
            volvoAsn.setProject(volvoLdjis.getMessageHeader());
            volvoAsn.setBeginningOfMessage(volvoLdjis.getBeginningOfMessage());
            volvoAsn.setSeller(volvoLdjis.getSeller());
            volvoAsn.setBuyer(volvoLdjis.getBuyer());
            volvoAsn.setShipFrom(volvoLdjis.getShipFrom());
            volvoAsn.setShipTo(volvoLdjis.getShipTo());
            volvoAsn.setAdviceNoteNo(volvoLdjis.getConsignmentReference());
            volvoAsn.setPlaceOfDischarge(volvoLdjis.getIdForPlaceOfDischarge());
            volvoAsn.setMtq(0.0);
            volvoAsn.setKgm(0.0);
            loading.setStatus(MesExtEnumUtil.MES_LOADING_STATUS.CREATE_ASN.getValue());
            ConvertBean.serviceModelInitialize(volvoAsn, AuthUtil.getSessionUser().getUserName());
            volvoAsnRDao.save(volvoAsn);

            int packageId = 1;
            for (MesLoadingListDetail detail : details) {
                ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
                DdlPreparedPack.getStringEqualPack(detail.getShippingCode(), "shippingCode", ddlPackBean);
                MesShippingOrderManagement orderManagement = shippingOrderManagementRepository.getByProperty(ddlPackBean);
                if (orderManagement != null) {
                    if (StringUtils.isEmpty(volvoAsn.getAsnNo())) {
                        ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
                        DdlPreparedPack.getStringEqualPack(orderManagement.getShippingCode(), "jisRackId", ddlPackBean);
                        MesCimVolvoJisRackId rackId = volvoJisRackIdRDao.getByProperty(ddlPackBean);
                        if (rackId != null) {
                            volvoAsn.setAsnNo(rackId.getAsnNo());
                        }
                    }

                    ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
                    DdlPreparedPack.getNumEqualPack(orderManagement.getId(), "pid", ddlPackBean);
                    List<MesShippingOrderManagementDetail> orderManagementDetails = shippingOrderManagementDetailRDao.findByHqlWhere(ddlPackBean);
                    for (MesShippingOrderManagementDetail orderManagementDetail : orderManagementDetails) {
                        MesCimVolvoJisRackId rackId = null;
                        MesCimVolvoJisRackIdDetail rackIdDetail = volvoJisRackIdDetailRDao.getById(orderManagementDetail.getSourceId());
                        if (rackIdDetail != null) {
                            rackId = volvoJisRackIdRDao.getById(rackIdDetail.getJisRackIdFid());
                        }
                        if (rackId == null) {
                            continue;
                        }
                        ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
                        DdlPreparedPack.getStringEqualPack(rackId.getConsignmentReference(), "consignmentReference", ddlPackBean);
                        DdlPreparedPack.getStringEqualPack(rackId.getJisRackId(), "rackId", ddlPackBean);
                        volvoLdjis = volvoLdjisRDao.getByProperty(ddlPackBean);
                        if (volvoLdjis == null) {
                            continue;
                        }

                        MesCimVolvoAsnDetail asnDetail = new MesCimVolvoAsnDetail();
                        asnDetail.setAsnId(volvoAsn.getId());
                        asnDetail.setSeq(orderManagement.getSeq());
                        asnDetail.setPackagingReference(volvoLdjis.getPackageReference());
                        asnDetail.setPackageTypeReference(volvoLdjis.getPackageReference());
                        asnDetail.setPackageId(String.valueOf(packageId));
                        asnDetail.setRackId(volvoLdjis.getRackId());
                        asnDetail.setLineItem(orderManagementDetail.getCustPartNo());
                        asnDetail.setDispatchedQuantity(volvoLdjis.getProductionReferencesSequenceNumber());
                        asnDetail.setFyon(volvoLdjis.getProductionReferencesFyon());
                        asnDetail.setRackPositionReferece(volvoLdjis.getProductionReferencesPosition());
                        asnDetail.setPurchaseOrderNumber(volvoLdjis.getOrderReference());
                        asnDetail.setAdditionalInternalDestination(volvoLdjis.getIdForAdditionalInternalDestination());
                        asnDetail.setOrganizeCode(organizeCode);
                        ConvertBean.serviceModelInitialize(asnDetail, AuthUtil.getSessionUser().getUserName());
                        volvoAsnDetailRDao.save(asnDetail);
                        packageId++;
                    }
                }
            }
        }
    }

    private List<MesLoadingList> getMesLoadingListByIds(List<Long> ids) {
        String organizeCode = AuthUtilExt.getOrganizeCode();
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(ids, "id", packBean);
        return baseRDao.findByHqlWhere(packBean);
    }

    private List<MesShippingOrderManagement> getMesShippingOrderManagement(MesLoadingList loading) {
        List<MesShippingOrderManagement> orderManagements = new ArrayList<>();
        String organizeCode = AuthUtilExt.getOrganizeCode();
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getNumEqualPack(loading.getId(), "pid", packBean);
        List<MesLoadingListDetail> details = listDetailRepository.findByHqlWhere(packBean);
        if (!CollectionUtils.isEmpty(details)) {
            for (MesLoadingListDetail detail : details) {
                DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
                DdlPreparedPack.getStringEqualPack(detail.getShippingCode(), "shippingCode", ddlPackBean);
                MesShippingOrderManagement orderManagement = shippingOrderManagementRepository.getByProperty(ddlPackBean);
                if (orderManagement != null) {
                    orderManagements.add(orderManagement);
                }
            }
        }
        return orderManagements;
    }
}
